/*
 * @(#)smtp.c	1.10 Aug.3,1993
 *
 * ޤ󡣤Ūʳλѡۤ¤ߤޤ
 * (C) yamasita@omronsoft.co.jp
 */

/*
 * This software is Copyright 1991 by Stan Barber. 
 *
 * Permission is hereby granted to copy, reproduce, redistribute or otherwise
 * use this software as long as: there is no monetary profit gained
 * specifically from the use or reproduction or this software, it is not
 * sold, rented, traded or otherwise marketed, and this copyright notice is
 * included prominently in any copy made. 
 *
 * The author make no claims as to the fitness or correctness of this software
 * for any use whatsoever, and it is provided as is. Any use of this software
 * is at the user's own risk. 
 *
 */

#ifdef	MSDOS
#ifdef	PCNFS
#include "pcnfs.h"
#endif
#include <stdio.h>
#include <sys/types.h>
#include <ctype.h>
#ifdef INETBIOS
#include	<stdlib.h>
#include	"inetbios.h"
#elif PATHWAY
#ifdef INTERNAL
#include <socket/types.h>
#include <socket/fcntl.h>
#include <socket/socket.h>
#include <socket/time.h>
#include <socket/in.h>
#include <socket/errno.h>
#include <socket/netdb.h>
#else
#include <sys/types.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/errno.h>
#include <netdb.h>
#endif
#else /* !INETBIOS !PATHWAY */
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#endif /* !INETBIOS */

#ifdef USG
# define	index	strchr
# define        bcopy(a,b,c)   memcpy(b,a,c)
# define        bzero(a,b)     memset(a,'\0',b)
#endif /* USG */

#include	"nntp.h"
#include	"gn.h"

#if	defined( INETBIOS ) || defined( PATHWAY )
int	sser_rd_fp;
int	sser_wr_fp;
#elif defined(PCNFS)
FILE	*sser_rd_fp;
FILE	*sser_wr_fp;
#else
extern FILE	*sser_rd_fp;
extern FILE	*sser_wr_fp;
#endif

static smtp_get_tcp_socket();	
	
/*
 * smtp_server_init  Get a connection to the remote smtp server.
 *
 *	Parameters:	"machine" is the machine to connect to.
 *
 *	Returns:	-1 on error
 *			0 on success.
 *
 *	Side effects:	Connects to server.
 *			"sser_rd_fp" and "sser_wr_fp" are fp's
 *			for reading and writing to server.
 */

smtp_server_init(machine)
char	*machine;
{
	int	sockt_rd, sockt_wr;
	char	line[256];

	sockt_rd = smtp_get_tcp_socket(machine);

	if (sockt_rd < 0)
		return (-1);

	/*
	 * Now we'll make file pointers (i.e., buffered I/O) out of
	 * the socket file descriptor.  Note that we can't just
	 * open a fp for reading and writing -- we have to open
	 * up two separate fp's, one for reading, one for writing.
	 */

#if	defined( INETBIOS ) || defined( PATHWAY )
	sser_rd_fp = sser_wr_fp = sockt_rd;
#else
	if ((sser_rd_fp = fdopen(sockt_rd, "r")) == NULL) {
		perror("server_init: fdopen #1");
		return (-1);
	}

	sockt_wr = dup(sockt_rd);
	if ((sser_wr_fp = fdopen(sockt_wr, "w")) == NULL) {
		perror("server_init: fdopen #2");
		sser_rd_fp = NULL;		/* from above */
		return (-1);
	}
#endif /* INETBIOS */

	/* Now get the server's signon message */

	(void) smtp_get_server(line, sizeof(line));
	return (atoi(line));
}


/*
 * smtp_get_tcp_socket -- get us a socket connected to the smtp server.
 *
 *	Parameters:	"machine" is the machine the server is running on.
 *
 *	Returns:	Socket connected to the news server if
 *			all is ok, else -1 on error.
 *
 *	Side effects:	Connects to server.
 *
 *	Errors:		Printed via perror.
 */
static
smtp_get_tcp_socket(machine)
char	*machine;	/* remote host */
{
	int	s;
	struct	sockaddr_in sin;
	struct	servent *getservbyname(), *sp;
	struct	hostent *gethostbyname(), *hp;
	unsigned long inet_addr();
	static struct hostent def;
	static struct in_addr defaddr;
	static char namebuf[ 256 ];

	if ((sp = getservbyname("smtp", "tcp")) ==  NULL) {
		fprintf(stderr, "smtp/tcp: Unknown service.\n");
		return (-1);
	}
	/* If not a raw ip address, try nameserver */
	if (!isdigit(*machine) ||
	    (long)(defaddr.s_addr = inet_addr(machine)) == -1)
		hp = gethostbyname(machine);
	else {
		/* Raw ip address, fake  */
		(void) strcpy(namebuf, machine);
		def.h_name = namebuf;
#ifdef INETBIOS
		def.h_addr = defaddr.s_addr;
#else
		def.h_addr = (char *)&defaddr;
#endif
		def.h_length = sizeof(struct in_addr);
		def.h_addrtype = AF_INET;
		def.h_aliases = 0;
		hp = &def;
	}
	if (hp == NULL) {
		fprintf(stderr, "%s: Unknown host.\n", machine);
		return (-1);
	}

	bzero((char *) &sin, sizeof(sin));
	sin.sin_family = hp->h_addrtype;
#ifdef INETBIOS
	sin.sin_port = htons( sp->s_port );
#else
	sin.sin_port = sp->s_port;
#endif

	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("socket");
		return (-1);
	}

	/* And then connect */

#ifdef INETBIOS
	sin.sin_addr.s_addr = hp->h_addr;
#else
	bcopy(hp->h_addr, (char *) &sin.sin_addr, hp->h_length);
#endif
	if (connect(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
		perror("connect");
		(void) close(s);
		return (-1);
	}

	return (s);
}


/*
 * smtp_put_server -- send a line of text to the server, terminating it
 * with CR and LF, as per ARPA standard.
 *
 *	Parameters:	"string" is the string to be sent to the
 *			server.
 *
 *	Returns:	Nothing.
 *
 *	Side effects:	Talks to the server.
 *
 *	Note:		This routine flushes the buffer each time
 *			it is called.  For large transmissions
 *			(i.e., posting news) don't use it.  Instead,
 *			do the fprintf's yourself, and then a final
 *			fflush.
 */

void
smtp_put_server(string)
char *string;
{
#if	defined( INETBIOS ) || defined( PATHWAY )
  	char	cp[128];
#endif
#ifdef DEBUG
	fprintf(stderr, ">>> %s\n", string);
#endif
#ifdef INETBIOS
	strcpy( cp, string );
	strcat( cp, "\r\n" );
	send( sser_wr_fp, cp, strlen( cp ), 0 );
#else
	fprintf(sser_wr_fp, "%s\r\n", string);
	(void) fflush(sser_wr_fp);
#endif
}


/*
 * smtp_get_server -- get a line of text from the server.  Strips
 * CR's and LF's.
 *
 *	Parameters:	"string" has the buffer space for the
 *			line received.
 *			"size" is the size of the buffer.
 *
 *	Returns:	-1 on error, 0 otherwise.
 *
 *	Side effects:	Talks to server, changes contents of "string".
 */

smtp_get_server(string, size)
char	*string;
int	size;
{
	register char *cp;
	char	*index();

#if	defined( INETBIOS ) || defined( PATHWAY )
	int	ret;
	if ( ( ret = recvs( sser_rd_fp, string, size, 0 ) ) < 0 ) {
		printf( "Return code %d\n", ret );
		return (-1);
	}
#else
	if (fgets(string, size, sser_rd_fp) == NULL)
		return (-1);
#endif

	if ((cp = index(string, '\r')) != NULL)
		*cp = '\0';
	else if ((cp = index(string, '\n')) != NULL)
		*cp = '\0';
#ifdef DEBUG
	fprintf(stderr, "<<< %s\n", string);
#endif

	return (0);
}


/*
 * smtp_close_server -- close the connection to the server, after sending
 *		the "quit" command.
 *
 *	Parameters:	None.
 *
 *	Returns:	Nothing.
 *
 *	Side effects:	Closes the connection with the server.
 *			You can't use "put_server" or "get_server"
 *			after this routine is called.
 */

void
smtp_close_server()
{
	char	sser_line[256];

#if	defined( INETBIOS ) || defined( PATHWAY )
	if (sser_wr_fp == 0 || sser_rd_fp == 0 )
#else
	if (sser_wr_fp == NULL || sser_rd_fp == NULL)
#endif
		return;

	smtp_put_server("QUIT");
	(void) smtp_get_server(sser_line, sizeof(sser_line));

#if	defined( INETBIOS ) || defined( PATHWAY )
#ifdef	DEBUG
	printf("soclose %d\n",soclose( sser_rd_fp ));
#else
	soclose( sser_rd_fp );
#endif
#else
	(void) fclose(sser_wr_fp);
	(void) fclose(sser_rd_fp);
#endif
}
#endif
